<?php
declare(strict_types=1);

namespace Inti\Elementor\Widget\Form;

use Elementor\Controls_Manager;
use Elementor\Core\Kits\Documents\Tabs\Global_Colors;
use Elementor\Core\Kits\Documents\Tabs\Global_Typography;
use Elementor\Group_Control_Typography;
use Elementor\Widget_Base;

/**
 * Class AbstractFormWidget
 *
 * @package Inti\Elementor\Widget\Form
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-elementor
 */
abstract class AbstractFormWidget extends Widget_Base
{
    private static int $instanceCount = 0;

    protected function get_element_id(): string
    {
        return $this->get_name() . '-' . (self::$instanceCount++);
    }

    public function get_style_depends(): array
    {
        return ['inti-widgets'];
    }

    protected function register_form_controls(): void
    {
        $this->start_controls_section('section_label', [
            'label' => 'Oznaka',
            'tab' => Controls_Manager::TAB_CONTENT,
        ]);

        $this->add_control('label', [
            'label' => 'Oznaka',
            'type' => Controls_Manager::TEXT,
        ]);

        $this->add_control('hide_label', [
            'label' => 'Sakrij oznaku',
            'type' => Controls_Manager::SWITCHER,
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_style_element', [
            'label' => 'Element',
            'tab' => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control('element_margin', [
            'label' => 'Razmak',
            'type' => Controls_Manager::SLIDER,
            'size_units' => ['px'],
            'default' => [
                'unit' => 'px',
                'size' => 10,
            ],
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'margin-bottom: {{SIZE}}{{UNIT}}',
            ],
        ]);

        $this->add_control('element_color', [
            'label' => 'Boja teksta',
            'type' => Controls_Manager::COLOR,
            'global' => [
                'default' => Global_Colors::COLOR_TEXT,
            ],
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'color: {{VALUE}}',
            ],
        ]);

        $this->add_group_control(Group_Control_Typography::get_type(), [
            'name' => 'element_typography',
            'selector' => '{{WRAPPER}} .inti-element-field',
            'global' => [
                'default' => Global_Typography::TYPOGRAPHY_TEXT,
            ],
        ]);

        $this->add_control('element_background_color', [
            'label' => 'Pozadinska boja',
            'type' => Controls_Manager::COLOR,
            'separator' => 'before',
            'default' => '#fff',
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'background-color: {{VALUE}}',
            ],
        ]);

        $this->add_control('element_border_color', [
            'label' => 'Boja obruba',
            'type' => Controls_Manager::COLOR,
            'separator' => 'before',
            'global' => [
                'default' => Global_Colors::COLOR_TEXT,
            ],
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'border-color: {{VALUE}}',
            ],
        ]);

        $this->add_control('element_border_width', [
            'label' => 'Debljina obruba',
            'type' => Controls_Manager::DIMENSIONS,
            'placeholder' => '1',
            'units' => ['px'],
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'border-width: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
            ],
        ]);

        $this->add_control('element_border_radius', [
            'label' => 'Zaobljenje rubova',
            'type' => Controls_Manager::DIMENSIONS,
            'placeholder' => '0',
            'units' => ['px', '%'],
            'selectors' => [
                '{{WRAPPER}} .inti-element-field' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
            ],
        ]);

        $this->end_controls_section();

        $this->start_controls_section('section_style_label', [
            'label' => 'Oznaka',
            'tab' => Controls_Manager::TAB_STYLE,
        ]);

        $this->add_control('element_label_margin', [
            'label' => 'Razmak',
            'type' => Controls_Manager::SLIDER,
            'size_units' => ['px'],
            'default' => [
                'unit' => 'px',
                'size' => 5,
            ],
            'selectors' => [
                '{{WRAPPER}} .inti-element-label' => 'margin-bottom: {{SIZE}}{{UNIT}}',
            ],
        ]);

        $this->add_control('element_label_color', [
            'label' => 'Boja teksta',
            'type' => Controls_Manager::COLOR,
            'global' => [
                'default' => Global_Colors::COLOR_TEXT,
            ],
            'selectors' => [
                '{{WRAPPER}} .inti-element-label' => 'color: {{VALUE}}',
            ],
        ]);

        $this->add_group_control(Group_Control_Typography::get_type(), [
            'name' => 'element_label_typography',
            'selector' => '{{WRAPPER}} .inti-element-label',
            'global' => [
                'default' => Global_Typography::TYPOGRAPHY_TEXT,
            ],
        ]);

        $this->end_controls_section();
    }

    protected function render(): void
    {
        $element_id = $this->get_element_id();

        $this->add_render_attribute([
            'element-wrapper' => [
                'class' => [
                    'inti-element',
                ],
            ],
            'element-label' => [
                'class' => [
                    'inti-element-label',
                ],
                'for' => $element_id,
            ],
            'field-wrapper' => [
                'class' => [
                    'inti-element-field-wrapper',
                ],
            ],
            'field' => [
                'class' => [
                    'inti-element-field',
                ],
                'id' => $element_id,
            ]
        ]);

        ?>
        <div <?= $this->get_render_attribute_string('element-wrapper'); ?>>
            <?php $this->render_label(); ?>
            <?php $this->render_element(); ?>
        </div>
        <?php
    }

    protected function render_label(): void
    {
        $settings = $this->get_settings_for_display();

        if ($settings['hide_label'] === 'yes' || empty($settings['label'])) {
            return;
        }

        ?>
        <label <?= $this->get_render_attribute_string('element-label'); ?>>
            <?= $settings['label']; ?>
        </label>
        <?php
    }

    protected abstract function render_element(): void;
}
